//-----------------------------------------------------------------------------
// A tool to force the clipboard format to plain text. This is useful when
// cutting and pasting between applications that supported formatted text,
// if you want to copy the text but not the formatting.
// Jonathan Westhues, June 2007
//-----------------------------------------------------------------------------
#include <windows.h>
#include <commctrl.h>
#include <process.h>
#include <stdio.h>
#include <stdlib.h>

static HINSTANCE    Instance;

static HWND         MainWindow;
static HWND         MainLabel;
static HFONT        MainFont;

static HWND         NextViewerInChain;

//-----------------------------------------------------------------------------
// Set the static control to show whatever text is on the clipboard, and
// then force the clipboard back to plain text.
//-----------------------------------------------------------------------------
static void UpdateForClipboardText(void)
{
    int i;
    for(i = 0; i < 20; i++) {
        if(OpenClipboard(MainWindow)) break;
        Sleep(100);
    }
    if(i >= 20) return;
    
    // So now we have the clipboard open. See if there's text there.
    if(!IsClipboardFormatAvailable(CF_TEXT)) {
        // No text, so nothing to do.
        CloseClipboard();
        return;
    }

    // There is text on the clipboard; let's retrieve it now.
    HANDLE from = GetClipboardData(CF_TEXT);
    char *s = (char *)GlobalLock(from);
    char *scp = (char *)malloc(strlen(s) + 10);
    strcpy(scp, s);
    GlobalUnlock(from);

    // Now delete everything on the clipboard, in all formats (including the
    // text, but we've saved that).
    EmptyClipboard();

    HANDLE to = GlobalAlloc(GMEM_MOVEABLE, strlen(scp) + 10);
    char *tos = (char *)GlobalLock(to);
    strcpy(tos, scp);
    GlobalUnlock(to);
    SetClipboardData(CF_TEXT, to);

    // And we're done with the clipboard.
    CloseClipboard();

    // Show the text in our window, not that it's particularly meaningful.
    unsigned max = 10000;
    if(strlen(scp) > max) {
        strcpy(scp+max, "...");
    }
    SendMessage(MainLabel, WM_SETTEXT, 0, (LPARAM)scp);
    free(scp);
}

static void Resized(void)
{
    RECT r;
    GetClientRect(MainWindow, &r);
    MoveWindow(MainLabel, 2, 2, r.right - 4, r.bottom - 4, TRUE);
}

//-----------------------------------------------------------------------------
// Draw the `where are we' information on-screen.
//-----------------------------------------------------------------------------
static LRESULT MainWndProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
    switch (msg) {
        case WM_SIZE:
            Resized();
            break;

        case WM_CHANGECBCHAIN: {
            HWND beingRemoved = (HWND)wParam;
            HWND afterBeingRemoved = (HWND)lParam;

            if(beingRemoved == NextViewerInChain) {
                NextViewerInChain = afterBeingRemoved;
            } else {
                if(NextViewerInChain) {
                    SendMessage(NextViewerInChain, msg, wParam, lParam);
                }
            }
            return 0;
        }

        case WM_DRAWCLIPBOARD:
            UpdateForClipboardText();

            if(NextViewerInChain) {
                SendMessage(NextViewerInChain, msg, wParam, lParam);
            }
            break;

        case WM_CLOSE:
        case WM_DESTROY:
            ChangeClipboardChain(MainWindow, NextViewerInChain);
            PostQuitMessage(0);
            break;

        default:
            return DefWindowProc(hwnd, msg, wParam, lParam);
    }

    return 1;
}

//-----------------------------------------------------------------------------
// Create our window class; nothing exciting.
//-----------------------------------------------------------------------------
static BOOL MakeWindowClass()
{
    WNDCLASSEX wc;
    memset(&wc, 0, sizeof(wc));
    wc.cbSize = sizeof(wc);

    wc.style            = CS_BYTEALIGNCLIENT | CS_BYTEALIGNWINDOW | CS_OWNDC |
                            CS_DBLCLKS;
    wc.lpfnWndProc      = (WNDPROC)MainWndProc;
    wc.hInstance        = Instance;
    wc.hbrBackground    = (HBRUSH)COLOR_BTNSHADOW;
    wc.lpszClassName    = "clipforce";
    wc.lpszMenuName     = NULL;
    wc.hCursor          = LoadCursor(NULL, IDC_ARROW);

    return RegisterClassEx(&wc);
}

//-----------------------------------------------------------------------------
// Create a window with a given client area.
//-----------------------------------------------------------------------------
static HWND CreateWindowClient(DWORD exStyle, char *className, char *windowName,
    DWORD style, int x, int y, int width, int height, HWND parent,
    HMENU menu, HINSTANCE instance, void *param)
{
    HWND h = CreateWindowEx(exStyle, className, windowName, style, x, y,
        width, height, parent, menu, instance, param);

    RECT r;
    GetClientRect(h, &r);
    width = width - (r.right - width);
    height = height - (r.bottom - height);
    
    SetWindowPos(h, HWND_TOP, x, y, width, height, 0);

    return h;
}

//-----------------------------------------------------------------------------
// Entry point into the program.
//-----------------------------------------------------------------------------
int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
    LPSTR lpCmdLine, INT nCmdShow)
{
    Instance = hInstance;

    MainFont = CreateFont(12, 0, 0, 0, FW_REGULAR, FALSE, FALSE, FALSE,
        ANSI_CHARSET, OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS,
        ANTIALIASED_QUALITY, FF_DONTCARE, "Lucida Console");

    MakeWindowClass();

    MainWindow = CreateWindowClient(0, "clipforce",
        "Force Clipboard to Plain Text",
        WS_OVERLAPPED | WS_SYSMENU | WS_SIZEBOX | WS_MINIMIZEBOX, 
        120, 120, 440, 130, NULL, NULL,
        Instance, NULL);

    MainLabel = CreateWindowEx(WS_EX_CLIENTEDGE, WC_EDIT, "",
        WS_CHILD | WS_CLIPSIBLINGS | WS_VISIBLE | ES_MULTILINE | ES_READONLY |
        ES_AUTOVSCROLL | ES_AUTOHSCROLL | WS_VSCROLL | WS_HSCROLL,
        2, 2, 196, 196, MainWindow, NULL, Instance, NULL);
    SendMessage(MainLabel, WM_SETFONT, (WPARAM)MainFont, TRUE);

    UpdateForClipboardText();

    NextViewerInChain = SetClipboardViewer(MainWindow);

    Resized();

    ShowWindow(MainWindow, SW_SHOW);

    MSG msg;
    DWORD ret;
    while(ret = GetMessage(&msg, NULL, 0, 0)) {
        TranslateMessage(&msg);
        DispatchMessage(&msg);
    }

    return 0;
}
